@@ -40,6 +40,10 @@ type logsOpts struct {
 	extr     bool
 }
 
+type cliResult interface {
+	HasError() bool
+}
+
 func Main() {
 	var (
 		pbmCmd       = kingpin.New("pbm", "Percona Backup for MongoDB")
@@ -62,13 +66,22 @@ func Main() {
 
 	backupCmd := pbmCmd.Command("backup", "Make backup")
 	backup := backupOpts{}
-	backupCmd.Flag("compression", "Compression type <none>/<gzip>/<snappy>/<lz4>/<s2>/<pgzip>").
+	backupCmd.Flag("compression", "Compression type <none>/<gzip>/<snappy>/<lz4>/<s2>/<pgzip>/<zstd>").
 		Default(string(pbm.CompressionTypeS2)).
 		EnumVar(&backup.compression,
 			string(pbm.CompressionTypeNone), string(pbm.CompressionTypeGZIP),
 			string(pbm.CompressionTypeSNAPPY), string(pbm.CompressionTypeLZ4),
 			string(pbm.CompressionTypeS2), string(pbm.CompressionTypePGZIP),
+			string(pbm.CompressionTypeZstandard),
 		)
+	backupCmd.Flag("type", fmt.Sprintf("backup type: <%s>/<%s>", pbm.PhysicalBackup, pbm.LogicalBackup)).
+		Default(string(pbm.LogicalBackup)).Short('t').
+		EnumVar(&backup.typ,
+			string(pbm.PhysicalBackup),
+			string(pbm.LogicalBackup),
+		)
+	backupCmd.Flag("compression-level", "Compression level (specific to the compression type)").
+		IntsVar(&backup.compressionLevel)
 
 	cancelBcpCmd := pbmCmd.Command("cancel-backup", "Cancel backup")
 
@@ -77,10 +90,18 @@ func Main() {
 	restoreCmd.Arg("backup_name", "Backup name to restore").StringVar(&restore.bcp)
 	restoreCmd.Flag("time", fmt.Sprintf("Restore to the point-in-time. Set in format %s", datetimeFormat)).StringVar(&restore.pitr)
 	restoreCmd.Flag("base-snapshot", "Override setting: Name of older snapshot that PITR will be based on during restore.").StringVar(&restore.pitrBase)
+	restoreCmd.Flag("wait", "Wait for the restore to finish.").Short('w').BoolVar(&restore.wait)
+
+	replayCmd := pbmCmd.Command("oplog-replay", "Replay oplog")
+	replayOpts := replayOptions{}
+	replayCmd.Flag("start", fmt.Sprintf("Replay oplog from the time. Set in format %s", datetimeFormat)).Required().StringVar(&replayOpts.start)
+	replayCmd.Flag("end", "Replay oplog to the time. Set in format %s").Required().StringVar(&replayOpts.end)
+	replayCmd.Flag("wait", "Wait for the restore to finish.").Short('w').BoolVar(&replayOpts.wait)
 
 	listCmd := pbmCmd.Command("list", "Backup list")
 	list := listOpts{}
 	listCmd.Flag("restore", "Show last N restores").Default("false").BoolVar(&list.restore)
+	listCmd.Flag("oplog-replay", "Show last N oplog replays").Default("false").BoolVar(&list.oplogReplay)
 	listCmd.Flag("full", "Show extended restore info").Default("false").Short('f').Hidden().BoolVar(&list.full)
 	listCmd.Flag("size", "Show last N backups").Default("0").IntVar(&list.size)
 
@@ -101,7 +122,7 @@ func Main() {
 	logsCmd.Flag("tail", "Show last N entries, 20 entries are shown by default, 0 for all logs").Short('t').Default("20").Int64Var(&logs.tail)
 	logsCmd.Flag("node", "Target node in format replset[/host:posrt]").Short('n').StringVar(&logs.node)
 	logsCmd.Flag("severity", "Severity level D, I, W, E or F, low to high. Choosing one includes higher levels too.").Short('s').Default("I").EnumVar(&logs.severity, "D", "I", "W", "E", "F")
-	logsCmd.Flag("event", "Event in format backup[/2020-10-06T11:45:14Z]. Events: backup, restore, cancelBackup, resyncBcpList, pitr, pitrestore, delete").Short('e').StringVar(&logs.event)
+	logsCmd.Flag("event", "Event in format backup[/2020-10-06T11:45:14Z]. Events: backup, restore, cancelBackup, resync, pitr, pitrestore, delete").Short('e').StringVar(&logs.event)
 	logsCmd.Flag("opid", "Operation ID").Short('i').StringVar(&logs.opid)
 	logsCmd.Flag("extra", "Show extra data in text format").Hidden().Short('x').BoolVar(&logs.extr)
 
@@ -143,6 +164,7 @@ func Main() {
 	if err != nil {
 		exitErr(errors.Wrap(err, "connect to mongodb"), pbmOutF)
 	}
+	pbmClient.InitLogger("", "")
 
 	switch cmd {
 	case configCmd.FullCommand():
@@ -154,6 +176,8 @@ func Main() {
 		out, err = cancelBcp(pbmClient)
 	case restoreCmd.FullCommand():
 		out, err = runRestore(pbmClient, &restore, pbmOutF)
+	case replayCmd.FullCommand():
+		out, err = replayOplog(pbmClient, replayOpts, pbmOutF)
 	case listCmd.FullCommand():
 		out, err = runList(pbmClient, &list)
 	case deleteBcpCmd.FullCommand():
@@ -171,6 +195,10 @@ func Main() {
 	}
 
 	printo(out, pbmOutF)
+
+	if r, ok := out.(cliResult); ok && r.HasError() {
+		os.Exit(1)
+	}
 }
 
 func printo(out fmt.Stringer, f outFormat) {
@@ -202,7 +230,7 @@ func exitErr(e error, f outFormat) {
 		var m interface{}
 		m = e
 		if _, ok := e.(json.Marshaler); !ok {
-			m = map[string]error{"Error": e}
+			m = map[string]string{"Error": e.Error()}
 		}
 		var err error
 		if f == outJSONpretty {
@@ -277,12 +305,13 @@ func runLogs(cn *pbm.PBM, l *logsOpts) (fmt.Stringer, error) {
 }
 
 type snapshotStat struct {
-	Name       string     `json:"name"`
-	Size       int64      `json:"size,omitempty"`
-	Status     pbm.Status `json:"status"`
-	Err        string     `json:"error,omitempty"`
-	StateTS    int64      `json:"completeTS"`
-	PBMVersion string     `json:"pbmVersion"`
+	Name       string         `json:"name"`
+	Size       int64          `json:"size,omitempty"`
+	Status     pbm.Status     `json:"status"`
+	Err        string         `json:"error,omitempty"`
+	StateTS    int64          `json:"completeTS"`
+	PBMVersion string         `json:"pbmVersion"`
+	Type       pbm.BackupType `json:"type"`
 }
 
 type pitrRange struct {
@@ -431,7 +460,6 @@ func lastLogErr(cn *pbm.PBM, op pbm.Command, after int64) (string, error) {
 				Event:    string(op),
 			},
 		}, 1)
-
 	if err != nil {
 		return "", errors.Wrap(err, "get log records")
 	}
