@@ -172,11 +172,16 @@ func (c cluster) String() (s string) {
 }
 
 func clusterStatus(cn *pbm.PBM, uri string) (fmt.Stringer, error) {
-	clstr, err := cn.ClusterMembers(nil)
+	clstr, err := cn.ClusterMembers()
 	if err != nil {
 		return nil, errors.Wrap(err, "get cluster members")
 	}
 
+	clusterTime, err := cn.ClusterTime()
+	if err != nil {
+		return nil, errors.Wrap(err, "read cluster time")
+	}
+
 	var ret cluster
 
 	for _, c := range clstr {
@@ -207,6 +212,10 @@ func clusterStatus(cn *pbm.PBM, uri string) (fmt.Stringer, error) {
 				nd.Errs = append(nd.Errs, fmt.Sprintf("ERROR: get agent status: %v", err))
 				continue
 			}
+			if stat.Heartbeat.T+pbm.StaleFrameSec < clusterTime.T {
+				nd.Errs = append(nd.Errs, fmt.Sprintf("ERROR: lost agent, last heartbeat: %v", stat.Heartbeat.T))
+				continue
+			}
 			nd.Ver = "v" + stat.Ver
 			nd.OK, nd.Errs = stat.OK()
 		}
@@ -295,7 +304,7 @@ func getPitrErr(cn *pbm.PBM) (string, error) {
 		return "", errors.Wrap(err, "get current epoch")
 	}
 
-	shards, err := cn.ClusterMembers(nil)
+	shards, err := cn.ClusterMembers()
 	if err != nil {
 		log.Fatalf("Error: get cluster members: %v", err)
 	}
@@ -457,7 +466,7 @@ func (s storageStat) String() string {
 		var status string
 		switch sn.Status {
 		case pbm.StatusDone:
-			status = fmt.Sprintf(" [complete: %s]", fmtTS(sn.StateTS))
+			status = fmt.Sprintf(" <%s> [complete: %s]", sn.Type, fmtTS(sn.StateTS))
 		case pbm.StatusCancelled:
 			status = fmt.Sprintf(" [!cancelled: %s]", fmtTS(sn.StateTS))
 		case pbm.StatusError:
@@ -515,7 +524,7 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 		return s, errors.Wrap(err, "define cluster state")
 	}
 
-	shards, err := cn.ClusterMembers(inf)
+	shards, err := cn.ClusterMembers()
 	if err != nil {
 		return s, errors.Wrap(err, "get cluster members")
 	}
@@ -540,19 +549,27 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 			Status:     bcp.Status,
 			StateTS:    bcp.LastTransitionTS,
 			PBMVersion: bcp.PBMVersion,
+			Type:       bcp.Type,
 		}
 
 		switch bcp.Status {
 		case pbm.StatusDone:
 			snpsht.StateTS = int64(bcp.LastWriteTS.T)
-			sz, err := getSnapshotSize(bcp.Replsets, stg)
+			var err error
+			switch bcp.Type {
+			case pbm.PhysicalBackup:
+				snpsht.Size, err = getPhysSnapshotSize(&bcp, stg)
+			default:
+				snpsht.Size, err = getSnapshotSize(bcp.Replsets, stg)
+			}
 			if err != nil {
 				snpsht.Err = err.Error()
 				snpsht.Status = pbm.StatusError
 			}
-			snpsht.Size = sz
 		case pbm.StatusError:
 			snpsht.Err = bcp.Error
+		case pbm.StatusCancelled:
+			// leave as it is, not to rewrite status with the `stuck` error
 		default:
 			if bcp.Hb.T+pbm.StaleFrameSec < now.T {
 				snpsht.Err = fmt.Sprintf("Backup stuck at `%v` stage, last beat ts: %d", bcp.Status, bcp.Hb.T)
@@ -571,7 +588,7 @@ func getStorageStat(cn *pbm.PBM) (fmt.Stringer, error) {
 }
 
 func getPITRranges(cn *pbm.PBM, stg storage.Storage) (*pitrRanges, error) {
-	shards, err := cn.ClusterMembers(nil)
+	shards, err := cn.ClusterMembers()
 	if err != nil {
 		return nil, errors.Wrap(err, "get cluster members")
 	}
@@ -641,8 +658,22 @@ func getSnapshotSize(rsets []pbm.BackupReplset, stg storage.Storage) (s int64, e
 		if err != nil {
 			return s, errors.Wrapf(err, "get file %s", rs.DumpName)
 		}
+		op, err := stg.FileStat(rs.OplogName)
+		if err != nil {
+			return s, errors.Wrapf(err, "get file %s", rs.OplogName)
+		}
 
-		s += ds.Size
+		s += ds.Size + op.Size
+	}
+
+	return s, nil
+}
+
+func getPhysSnapshotSize(bcp *pbm.BackupMeta, stg storage.Storage) (s int64, err error) {
+	for _, rs := range bcp.Replsets {
+		for _, f := range rs.Files {
+			s += f.StgSize
+		}
 	}
 
 	return s, nil
