@@ -3,10 +3,12 @@ package sharded
 import (
 	"log"
 
+	pbmt "github.com/percona/percona-backup-mongodb/pbm"
+
 	"github.com/percona/percona-backup-mongodb/e2e-tests/pkg/pbm"
 )
 
-func (c *Cluster) ClockSkew() {
+func (c *Cluster) ClockSkew(typ pbmt.BackupType) {
 	timeShifts := []string{
 		"+90m", "-195m", "+2d", "-7h", "+11m", "+42d", "-13h",
 	}
@@ -26,12 +28,8 @@ func (c *Cluster) ClockSkew() {
 			log.Fatalf("ERROR: clock skew for %v: %v\n", rs, err)
 		}
 
-		log.Println("[START] Basic Backup & Restore / ClockSkew", rs, shift)
-		c.BackupAndRestore()
-		log.Println("[DONE] Basic Backup & Restore / ClockSkew", rs, shift)
-
-		log.Println("[START] Backup Data Bounds Check / ClockSkew", rs, shift)
-		c.BackupBoundsCheck()
-		log.Println("[DONE] Backup Data Bounds Check / ClockSkew", rs, shift)
+		log.Printf("[START] %s Backup Data Bounds Check / ClockSkew %s %s", typ, rs, shift)
+		c.BackupBoundsCheck(typ)
+		log.Printf("[DONE] %s Backup Data Bounds Check / ClockSkew %s %s", typ, rs, shift)
 	}
 }
