@@ -5,14 +5,9 @@ import (
 	"context"
 	"encoding/json"
 	"io"
-	"log"
 	"time"
 
-	"github.com/mongodb/mongo-tools/common/db"
 	mlog "github.com/mongodb/mongo-tools/common/log"
-	"github.com/mongodb/mongo-tools/common/options"
-	"github.com/mongodb/mongo-tools/common/progress"
-	"github.com/mongodb/mongo-tools/mongodump"
 	"github.com/pkg/errors"
 	"go.mongodb.org/mongo-driver/bson/primitive"
 	"go.mongodb.org/mongo-driver/mongo"
@@ -34,18 +29,23 @@ func init() {
 type Backup struct {
 	cn   *pbm.PBM
 	node *pbm.Node
+	typ  pbm.BackupType
 }
 
 func New(cn *pbm.PBM, node *pbm.Node) *Backup {
 	return &Backup{
 		cn:   cn,
 		node: node,
+		typ:  pbm.LogicalBackup,
 	}
 }
 
-// Run runs the backup
-func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
-	return b.run(ctx, bcp, opid, l)
+func NewPhysical(cn *pbm.PBM, node *pbm.Node) *Backup {
+	return &Backup{
+		cn:   cn,
+		node: node,
+		typ:  pbm.PhysicalBackup,
+	}
 }
 
 func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMode) error {
@@ -55,6 +55,7 @@ func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMod
 	}
 
 	meta := &pbm.BackupMeta{
+		Type:           b.typ,
 		OPID:           opid.String(),
 		Name:           bcp.Name,
 		Compression:    bcp.Compression,
@@ -85,20 +86,16 @@ func (b *Backup) Init(bcp pbm.BackupCmd, opid pbm.OPID, balancer pbm.BalancerMod
 	return b.cn.SetBackupMeta(meta)
 }
 
-// run the backup.
+// Run runs backup.
 // TODO: describe flow
-func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
+func (b *Backup) Run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *plog.Event) (err error) {
 	inf, err := b.node.GetInfo()
 	if err != nil {
 		return errors.Wrap(err, "get cluster info")
 	}
 
-	rsName := inf.SetName
-
 	rsMeta := pbm.BackupReplset{
-		Name:         rsName,
-		OplogName:    getDstName("oplog", bcp, inf.SetName),
-		DumpName:     getDstName("dump", bcp, inf.SetName),
+		Name:         inf.SetName,
 		StartTS:      time.Now().UTC().Unix(),
 		Status:       pbm.StatusRunning,
 		Conditions:   []pbm.Condition{},
@@ -115,7 +112,7 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		return errors.Wrap(err, "balancer status, get backup meta")
 	}
 
-	// on any error the RS' and the backup' (in case this is the backup leader) meta will be marked aproprietly
+	// on any error the RS' and the backup' (in case this is the backup leader) meta will be marked appropriately
 	defer func() {
 		if err != nil {
 			status := pbm.StatusError
@@ -128,7 +125,7 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 					Replsets: []pbm.BackupReplset{rsMeta},
 				}
 
-				l.Info("delete artefacts from storage: %v", b.cn.DeleteBackupFiles(meta, stg))
+				l.Info("delete artifacts from storage: %v", b.cn.DeleteBackupFiles(meta, stg))
 			}
 
 			ferr := b.cn.ChangeRSState(bcp.Name, rsMeta.Name, status, err.Error())
@@ -199,130 +196,18 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 		return errors.Wrap(err, "waiting for start")
 	}
 
-	oplog := NewOplog(b.node)
-	oplogTS, err := oplog.LastWrite()
+	switch b.typ {
+	case pbm.LogicalBackup:
+		err = b.doLogical(ctx, bcp, opid, &rsMeta, inf, stg, l)
+	case pbm.PhysicalBackup:
+		err = b.doPhysical(ctx, bcp, opid, &rsMeta, inf, stg, l)
+	default:
+		return errors.New("undefined backup type")
+	}
 	if err != nil {
-		return errors.Wrap(err, "define oplog start position")
+		return err
 	}
 
-	rsMeta.Status = pbm.StatusRunning
-	rsMeta.FirstWriteTS = oplogTS
-	err = b.cn.AddRSMeta(bcp.Name, rsMeta)
-	if err != nil {
-		return errors.Wrap(err, "add shard's metadata")
-	}
-
-	if inf.IsLeader() {
-		err := b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusRunning, inf, &pbm.WaitBackupStart)
-		if err != nil {
-			if errors.Cause(err) == errConvergeTimeOut {
-				return errors.Wrap(err, "couldn't get response from all shards")
-			}
-			return errors.Wrap(err, "check cluster for backup started")
-		}
-
-		err = b.setClusterFirstWrite(bcp.Name)
-		if err != nil {
-			return errors.Wrap(err, "set cluster first write ts")
-		}
-	}
-
-	// Waiting for cluster's StatusRunning to move further.
-	err = b.waitForStatus(bcp.Name, pbm.StatusRunning, nil)
-	if err != nil {
-		return errors.Wrap(err, "waiting for running")
-	}
-
-	// Save users and roles to the tmp collections so the restore would copy that data
-	// to the system collections. Have to do this because of issues with the restore and preserverUUID.
-	// see: https://jira.percona.com/browse/PBM-636 and comments
-	lw, err := b.node.CopyUsersNRolles()
-	if err != nil {
-		return errors.Wrap(err, "copy users and roles for the restore")
-	}
-
-	defer func() {
-		l.Info("dropping tmp collections")
-		err := b.node.DropTMPcoll()
-		if err != nil {
-			l.Warning("drop tmp users and roles: %v", err)
-		}
-	}()
-
-	// before proceeding any further we have to be sure that tmp users and roles
-	// have replicated to the node we're about to take a backup from
-	// *copying made on a primary but backup does a secondary node
-	l.Debug("wait for tmp users %v", lw)
-	err = b.node.WaitForWrite(lw)
-	if err != nil {
-		return errors.Wrap(err, "wait for tmp users and roles replication")
-	}
-
-	sz, err := b.node.SizeDBs()
-	if err != nil {
-		return errors.Wrap(err, "mongodump")
-	}
-	// if backup wouldn't be compressed we're assuming
-	// that the dump size could be up to 4x lagrer due to
-	// mongo's wieredtiger compression
-	if bcp.Compression == pbm.CompressionTypeNone {
-		sz *= 4
-	}
-
-	dump, err := newDump(b.node.ConnURI(), b.node.DumpConns())
-	if err != nil {
-		return errors.Wrap(err, "init mongodump options")
-	}
-	_, err = Upload(ctx, dump, stg, bcp.Compression, rsMeta.DumpName, sz)
-	if err != nil {
-		return errors.Wrap(err, "mongodump")
-	}
-	l.Info("mongodump finished, waiting for the oplog")
-
-	err = b.cn.ChangeRSState(bcp.Name, rsMeta.Name, pbm.StatusDumpDone, "")
-	if err != nil {
-		return errors.Wrap(err, "set shard's StatusDumpDone")
-	}
-
-	lwts, err := oplog.LastWrite()
-	if err != nil {
-		return errors.Wrap(err, "get shard's last write ts")
-	}
-
-	err = b.cn.SetRSLastWrite(bcp.Name, rsMeta.Name, lwts)
-	if err != nil {
-		return errors.Wrap(err, "set shard's last write ts")
-	}
-
-	if inf.IsLeader() {
-		err := b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusDumpDone, inf, nil)
-		if err != nil {
-			return errors.Wrap(err, "check cluster for dump done")
-		}
-
-		err = b.setClusterLastWrite(bcp.Name)
-		if err != nil {
-			return errors.Wrap(err, "set cluster last write ts")
-		}
-	}
-
-	err = b.waitForStatus(bcp.Name, pbm.StatusDumpDone, nil)
-	if err != nil {
-		return errors.Wrap(err, "waiting for dump done")
-	}
-
-	fwTS, lwTS, err := b.waitForFirstLastWrite(bcp.Name)
-	if err != nil {
-		return errors.Wrap(err, "get cluster first & last write ts")
-	}
-
-	l.Debug("set oplog span to %v / %v", fwTS, lwTS)
-	oplog.SetTailingSpan(fwTS, lwTS)
-	// size -1 - we're assuming oplog never exceed 97Gb (see comments in s3.Save method)
-	_, err = Upload(ctx, oplog, stg, bcp.Compression, rsMeta.OplogName, -1)
-	if err != nil {
-		return errors.Wrap(err, "oplog")
-	}
 	err = b.cn.ChangeRSState(bcp.Name, rsMeta.Name, pbm.StatusDone, "")
 	if err != nil {
 		return errors.Wrap(err, "set shard's StatusDone")
@@ -336,7 +221,7 @@ func (b *Backup) run(ctx context.Context, bcp pbm.BackupCmd, opid pbm.OPID, l *p
 			l.Debug("epoch set to %v", epch)
 		}
 
-		err = b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusDone, inf, nil)
+		err = b.reconcileStatus(bcp.Name, opid.String(), pbm.StatusDone, nil)
 		if err != nil {
 			return errors.Wrap(err, "check cluster for backup done")
 		}
@@ -423,6 +308,7 @@ func (rwe rwErr) Error() string {
 
 	return r
 }
+
 func (rwe rwErr) nil() bool {
 	return rwe.read == nil && rwe.compress == nil && rwe.write == nil
 }
@@ -435,22 +321,25 @@ type Source interface {
 var ErrCancelled = errors.New("backup canceled")
 
 // Upload writes data to dst from given src and returns an amount of written bytes
-func Upload(ctx context.Context, src Source, dst storage.Storage, compression pbm.CompressionType, fname string, sizeb int) (int64, error) {
+func Upload(ctx context.Context, src Source, dst storage.Storage, compression pbm.CompressionType, compressLevel *int, fname string, sizeb int) (int64, error) {
 	r, pw := io.Pipe()
 
-	w := Compress(pw, compression)
+	w, err := Compress(pw, compression, compressLevel)
+	if err != nil {
+		return 0, err
+	}
 
-	var err rwErr
+	var rwErr rwErr
 	var n int64
 	go func() {
-		n, err.read = src.WriteTo(w)
-		err.compress = w.Close()
+		n, rwErr.read = src.WriteTo(w)
+		rwErr.compress = w.Close()
 		pw.Close()
 	}()
 
 	saveDone := make(chan struct{})
 	go func() {
-		err.write = dst.Save(fname, r, sizeb)
+		rwErr.write = dst.Save(fname, r, sizeb)
 		saveDone <- struct{}{}
 	}()
 
@@ -466,15 +355,15 @@ func Upload(ctx context.Context, src Source, dst storage.Storage, compression pb
 
 	r.Close()
 
-	if !err.nil() {
-		return 0, err
+	if !rwErr.nil() {
+		return 0, rwErr
 	}
 
 	return n, nil
 }
 
-func (b *Backup) reconcileStatus(bcpName, opid string, status pbm.Status, ninf *pbm.NodeInfo, timeout *time.Duration) error {
-	shards, err := b.cn.ClusterMembers(ninf)
+func (b *Backup) reconcileStatus(bcpName, opid string, status pbm.Status, timeout *time.Duration) error {
+	shards, err := b.cn.ClusterMembers()
 	if err != nil {
 		return errors.Wrap(err, "get cluster members")
 	}
@@ -714,95 +603,3 @@ func (b *Backup) setClusterLastWrite(bcpName string) error {
 	err = b.cn.SetLastWrite(bcpName, lw)
 	return errors.Wrap(err, "set timestamp")
 }
-
-type mdump struct {
-	opts  *options.ToolOptions
-	conns int
-}
-
-func newDump(curi string, conns int) (*mdump, error) {
-	if conns <= 0 {
-		conns = 1
-	}
-
-	var err error
-
-	opts := options.New("mongodump", "0.0.1", "none", "", true, options.EnabledOptions{Auth: true, Connection: true, Namespace: true, URI: true})
-	opts.URI, err = options.NewURI(curi)
-	if err != nil {
-		return nil, errors.Wrap(err, "parse connection string")
-	}
-
-	err = opts.NormalizeOptionsAndURI()
-	if err != nil {
-		return nil, errors.Wrap(err, "parse opts")
-	}
-
-	opts.Direct = true
-
-	return &mdump{
-		opts:  opts,
-		conns: conns,
-	}, nil
-}
-
-// "logger" for the mongodup's ProgressManager.
-// need it to be able to write new progress data in a new line
-type progressWriter struct{}
-
-func (*progressWriter) Write(m []byte) (int, error) {
-	log.Printf("%s", m)
-	return len(m), nil
-}
-
-// Write always return 0 as written bytes. Needed to satisfy interface
-func (d *mdump) WriteTo(w io.Writer) (int64, error) {
-	pm := progress.NewBarWriter(&progressWriter{}, time.Second*60, 24, false)
-	mdump := mongodump.MongoDump{
-		ToolOptions: d.opts,
-		OutputOptions: &mongodump.OutputOptions{
-			// Archive = "-" means, for mongodump, use the provided Writer
-			// instead of creating a file. This is not clear at plain sight,
-			// you nee to look the code to discover it.
-			Archive:                "-",
-			NumParallelCollections: d.conns,
-		},
-		InputOptions:    &mongodump.InputOptions{},
-		SessionProvider: &db.SessionProvider{},
-		OutputWriter:    w,
-		ProgressManager: pm,
-	}
-	err := mdump.Init()
-	if err != nil {
-		return 0, errors.Wrap(err, "init")
-	}
-	pm.Start()
-	defer pm.Stop()
-
-	err = mdump.Dump()
-
-	return 0, errors.Wrap(err, "make dump")
-}
-
-func getDstName(typ string, bcp pbm.BackupCmd, rsName string) string {
-	name := bcp.Name
-
-	if rsName != "" {
-		name += "_" + rsName
-	}
-
-	name += "." + typ
-
-	switch bcp.Compression {
-	case pbm.CompressionTypeGZIP, pbm.CompressionTypePGZIP:
-		name += ".gz"
-	case pbm.CompressionTypeLZ4:
-		name += ".lz4"
-	case pbm.CompressionTypeSNAPPY:
-		name += ".snappy"
-	case pbm.CompressionTypeS2:
-		name += ".s2"
-	}
-
-	return name
-}
