@@ -5,6 +5,7 @@ import (
 	"context"
 	"fmt"
 	"net/url"
+	"os"
 	"strconv"
 	"strings"
 	"time"
@@ -43,11 +44,11 @@ const (
 	RestoresCollection = "pbmRestores"
 	// CmdStreamCollection is the name of the mongo collection that contains backup/restore commands stream
 	CmdStreamCollection = "pbmCmd"
-	//PITRChunksCollection contains index metadata of PITR chunks
+	// PITRChunksCollection contains index metadata of PITR chunks
 	PITRChunksCollection = "pbmPITRChunks"
-	//PITRChunksOldCollection contains archived index metadata of PITR chunks
+	// PITRChunksOldCollection contains archived index metadata of PITR chunks
 	PITRChunksOldCollection = "pbmPITRChunks.old"
-	// PBMOpLogCollection contains log of aquired locks (hence run ops)
+	// PBMOpLogCollection contains log of acquired locks (hence run ops)
 	PBMOpLogCollection = "pbmOpLog"
 	// AgentsStatusCollection is an agents registry with its status/health checks
 	AgentsStatusCollection = "pbmAgents"
@@ -63,15 +64,16 @@ var ErrNotFound = errors.New("not found")
 type Command string
 
 const (
-	CmdUndefined        Command = ""
-	CmdBackup           Command = "backup"
-	CmdRestore          Command = "restore"
-	CmdCancelBackup     Command = "cancelBackup"
-	CmdResyncBackupList Command = "resyncBcpList"
-	CmdPITR             Command = "pitr"
-	CmdPITRestore       Command = "pitrestore"
-	CmdDeleteBackup     Command = "delete"
-	CmdDeletePITR       Command = "deletePitr"
+	CmdUndefined    Command = ""
+	CmdBackup       Command = "backup"
+	CmdRestore      Command = "restore"
+	CmdReplay       Command = "replay"
+	CmdCancelBackup Command = "cancelBackup"
+	CmdResync       Command = "resync"
+	CmdPITR         Command = "pitr"
+	CmdPITRestore   Command = "pitrestore"
+	CmdDeleteBackup Command = "delete"
+	CmdDeletePITR   Command = "deletePitr"
 )
 
 func (c Command) String() string {
@@ -80,9 +82,11 @@ func (c Command) String() string {
 		return "Snapshot backup"
 	case CmdRestore:
 		return "Snapshot restore"
+	case CmdReplay:
+		return "Oplog replay"
 	case CmdCancelBackup:
-		return "Backup cancelation"
-	case CmdResyncBackupList:
+		return "Backup cancellation"
+	case CmdResync:
 		return "Resync storage"
 	case CmdPITR:
 		return "PITR incremental backup"
@@ -103,6 +107,7 @@ type Cmd struct {
 	Cmd        Command         `bson:"cmd"`
 	Backup     BackupCmd       `bson:"backup,omitempty"`
 	Restore    RestoreCmd      `bson:"restore,omitempty"`
+	Replay     ReplayCmd       `bson:"replay,omitempty"`
 	PITRestore PITRestoreCmd   `bson:"pitrestore,omitempty"`
 	Delete     DeleteBackupCmd `bson:"delete,omitempty"`
 	DeletePITR DeletePITRCmd   `bson:"deletePitr,omitempty"`
@@ -153,12 +158,20 @@ func (c Cmd) String() string {
 }
 
 type BackupCmd struct {
-	Name        string          `bson:"name"`
-	Compression CompressionType `bson:"compression"`
+	Type             BackupType      `bson:"type"`
+	Name             string          `bson:"name"`
+	Compression      CompressionType `bson:"compression"`
+	CompressionLevel *int            `bson:"level,omitempty"`
 }
 
 func (b BackupCmd) String() string {
-	return fmt.Sprintf("name: %s, compression: %s", b.Name, b.Compression)
+	var level string
+	if b.CompressionLevel == nil {
+		level = "default"
+	} else {
+		level = strconv.Itoa(*b.CompressionLevel)
+	}
+	return fmt.Sprintf("name: %s, compression: %s (level: %s)", b.Name, b.Compression, level)
 }
 
 type RestoreCmd struct {
@@ -170,9 +183,20 @@ func (r RestoreCmd) String() string {
 	return fmt.Sprintf("name: %s, backup name: %s", r.Name, r.BackupName)
 }
 
+type ReplayCmd struct {
+	Name  string              `bson:"name"`
+	Start primitive.Timestamp `bson:"start,omitempty"`
+	End   primitive.Timestamp `bson:"end,omitempty"`
+}
+
+func (c ReplayCmd) String() string {
+	return fmt.Sprintf("name: %s, time: %d - %d", c.Name, c.Start, c.End)
+}
+
 type PITRestoreCmd struct {
 	Name string `bson:"name"`
 	TS   int64  `bson:"ts"`
+	I    int64  `bson:"i"`
 	Bcp  string `bson:"bcp"`
 }
 
@@ -199,14 +223,66 @@ func (d DeleteBackupCmd) String() string {
 type CompressionType string
 
 const (
-	CompressionTypeNone   CompressionType = "none"
-	CompressionTypeGZIP   CompressionType = "gzip"
-	CompressionTypePGZIP  CompressionType = "pgzip"
-	CompressionTypeSNAPPY CompressionType = "snappy"
-	CompressionTypeLZ4    CompressionType = "lz4"
-	CompressionTypeS2     CompressionType = "s2"
+	CompressionTypeNone      CompressionType = "none"
+	CompressionTypeGZIP      CompressionType = "gzip"
+	CompressionTypePGZIP     CompressionType = "pgzip"
+	CompressionTypeSNAPPY    CompressionType = "snappy"
+	CompressionTypeLZ4       CompressionType = "lz4"
+	CompressionTypeS2        CompressionType = "s2"
+	CompressionTypeZstandard CompressionType = "zstd"
 )
 
+func isValidCompressionType(s string) bool {
+	switch CompressionType(s) {
+	case
+		CompressionTypeNone,
+		CompressionTypeGZIP,
+		CompressionTypePGZIP,
+		CompressionTypeSNAPPY,
+		CompressionTypeLZ4,
+		CompressionTypeS2,
+		CompressionTypeZstandard:
+		return true
+	}
+
+	return false
+}
+
+func (c CompressionType) Suffix() string {
+	switch c {
+	case CompressionTypeGZIP, CompressionTypePGZIP:
+		return ".gz"
+	case CompressionTypeLZ4:
+		return ".lz4"
+	case CompressionTypeSNAPPY:
+		return ".snappy"
+	case CompressionTypeS2:
+		return ".s2"
+	case CompressionTypeZstandard:
+		return ".zst"
+	default:
+		return ""
+	}
+}
+
+// FileCompression return compression alg based on given file extension
+func FileCompression(ext string) CompressionType {
+	switch ext {
+	default:
+		return CompressionTypeNone
+	case "gz":
+		return CompressionTypePGZIP
+	case "lz4":
+		return CompressionTypeLZ4
+	case "snappy":
+		return CompressionTypeSNAPPY
+	case "s2":
+		return CompressionTypeS2
+	case "zst":
+		return CompressionTypeZstandard
+	}
+}
+
 const (
 	PITRcheckRange       = time.Second * 15
 	AgentsStatCheckRange = time.Second * 5
@@ -251,7 +327,7 @@ func New(ctx context.Context, uri, appName string) (*PBM, error) {
 		return nil, errors.Wrap(err, "get topology")
 	}
 
-	if !inf.IsSharded() || inf.ReplsetRole() == ReplRoleConfigSrv {
+	if !inf.IsSharded() || inf.ReplsetRole() == RoleConfigSrv {
 		return pbm, errors.Wrap(pbm.setupNewDB(), "setup a new backups db")
 	}
 
@@ -378,7 +454,7 @@ func (p *PBM) setupNewDB() error {
 		return errors.Wrapf(err, "ensure lock index on %s", LockOpCollection)
 	}
 
-	// create indexs for the pitr cunks
+	// create indexs for the pitr chunks
 	_, err = p.Conn.Database(DB).Collection(PITRChunksCollection).Indexes().CreateMany(
 		p.ctx,
 		[]mongo.IndexModel{
@@ -439,8 +515,16 @@ func connect(ctx context.Context, uri, appName string) (*mongo.Client, error) {
 	return client, nil
 }
 
+type BackupType string
+
+const (
+	PhysicalBackup BackupType = "physical"
+	LogicalBackup  BackupType = "logical"
+)
+
 // BackupMeta is a backup's metadata
 type BackupMeta struct {
+	Type             BackupType           `bson:"type" json:"type"`
 	OPID             string               `bson:"opid" json:"opid"`
 	Name             string               `bson:"name" json:"name"`
 	Replsets         []BackupReplset      `bson:"replsets" json:"replsets"`
@@ -476,8 +560,9 @@ type Condition struct {
 
 type BackupReplset struct {
 	Name             string              `bson:"name" json:"name"`
-	DumpName         string              `bson:"dump_name" json:"backup_name" `
-	OplogName        string              `bson:"oplog_name" json:"oplog_name"`
+	Files            []File              `bson:"files,omitempty" json:"files,omitempty" `
+	DumpName         string              `bson:"dump_name,omitempty" json:"backup_name,omitempty" `
+	OplogName        string              `bson:"oplog_name,omitempty" json:"oplog_name,omitempty"`
 	StartTS          int64               `bson:"start_ts" json:"start_ts"`
 	Status           Status              `bson:"status" json:"status"`
 	LastTransitionTS int64               `bson:"last_transition_ts" json:"last_transition_ts"`
@@ -487,10 +572,20 @@ type BackupReplset struct {
 	Conditions       []Condition         `bson:"conditions" json:"conditions"`
 }
 
+type File struct {
+	Name    string      `bson:"filename" json:"filename"`
+	Size    int64       `bson:"fileSize" json:"fileSize"`
+	StgSize int64       `bson:"stgSize" json:"stgSize"`
+	Fmode   os.FileMode `bson:"fmode" json:"fmode"`
+}
+
 // Status is a backup current status
 type Status string
 
 const (
+	StatusInit  Status = "init"
+	StatusReady Status = "ready"
+
 	StatusStarting  Status = "starting"
 	StatusRunning   Status = "running"
 	StatusDumpDone  Status = "dumpDone"
@@ -525,9 +620,11 @@ func (b *BackupMeta) RS(name string) *BackupReplset {
 func (p *PBM) ChangeBackupStateOPID(opid string, s Status, msg string) error {
 	return p.changeBackupState(bson.D{{"opid", opid}}, s, msg)
 }
+
 func (p *PBM) ChangeBackupState(bcpName string, s Status, msg string) error {
 	return p.changeBackupState(bson.D{{"name", bcpName}}, s, msg)
 }
+
 func (p *PBM) changeBackupState(clause bson.D, s Status, msg string) error {
 	ts := time.Now().UTC().Unix()
 	_, err := p.Conn.Database(DB).Collection(BcpCollection).UpdateOne(
@@ -616,6 +713,18 @@ func (p *PBM) ChangeRSState(bcpName string, rsName string, s Status, msg string)
 	return err
 }
 
+func (p *PBM) RSSetPhyFiles(bcpName string, rsName string, f []File) error {
+	_, err := p.Conn.Database(DB).Collection(BcpCollection).UpdateOne(
+		p.ctx,
+		bson.D{{"name", bcpName}, {"replsets.name", rsName}},
+		bson.D{
+			{"$set", bson.M{"replsets.$.files": f}},
+		},
+	)
+
+	return err
+}
+
 func (p *PBM) SetRSLastWrite(bcpName string, rsName string, ts primitive.Timestamp) error {
 	_, err := p.Conn.Database(DB).Collection(BcpCollection).UpdateOne(
 		p.ctx,
@@ -658,7 +767,10 @@ func (p *PBM) GetLastBackup(before *primitive.Timestamp) (*BackupMeta, error) {
 }
 
 func (p *PBM) getRecentBackup(before *primitive.Timestamp, sort int) (*BackupMeta, error) {
-	q := bson.D{{"status", StatusDone}}
+	q := bson.D{
+		{"status", StatusDone},
+		{"type", bson.M{"$ne": string(PhysicalBackup)}},
+	}
 	if before != nil {
 		q = append(q, bson.E{"last_write_ts", bson.M{"$lte": before}})
 	}
@@ -720,6 +832,9 @@ func (p *PBM) BackupsList(limit int64) ([]BackupMeta, error) {
 		if err != nil {
 			return nil, errors.Wrap(err, "message decode")
 		}
+		if b.Type == "" {
+			b.Type = LogicalBackup
+		}
 		backups = append(backups, b)
 	}
 
@@ -758,15 +873,12 @@ func (p *PBM) BackupsDoneList(after *primitive.Timestamp, limit int64, order int
 
 // ClusterMembers returns list of replicasets current cluster consists of
 // (shards + configserver). The list would consist of on rs if cluster is
-// a non-sharded rs. If `inf` is nil, method would request mongo to define it.
-func (p *PBM) ClusterMembers(inf *NodeInfo) ([]Shard, error) {
-	var err error
-
-	if inf == nil {
-		inf, err = p.GetNodeInfo()
-		if err != nil {
-			return nil, errors.Wrap(err, "define cluster state")
-		}
+// a non-sharded rs.
+func (p *PBM) ClusterMembers() ([]Shard, error) {
+	// it would be a config server in sharded cluster
+	inf, err := p.GetNodeInfo()
+	if err != nil {
+		return nil, errors.Wrap(err, "define cluster state")
 	}
 
 	shards := []Shard{{
@@ -916,20 +1028,6 @@ func (e Epoch) TS() primitive.Timestamp {
 	return primitive.Timestamp(e)
 }
 
-// FileCompression return compression alg based on given file extention
-func FileCompression(ext string) CompressionType {
-	switch ext {
-	default:
-		return CompressionTypeNone
-	case "gz":
-		return CompressionTypePGZIP
-	case "lz4":
-		return CompressionTypeLZ4
-	case "snappy":
-		return CompressionTypeS2
-	}
-}
-
 // CopyColl copy documents matching the given filter and return number of copied documents
 func CopyColl(ctx context.Context, from, to *mongo.Collection, filter interface{}) (n int, err error) {
 	cur, err := from.Find(ctx, filter)
